use hercules_cg::{ChipDesc, FuOp, FunctionalUnit, SliceDesc, Switchbox};

static mut CHIP: ChipDesc<8, 8, 1> = chip_init();
static mut FRAME_IDX: usize = 0;

static mut INPUT_DATA: [i16; 16] = [0; 16];

pub const fn chip_init() -> ChipDesc<8, 8, 1> {
    let functional_unit = FunctionalUnit {
        op_type: FuOp::Default,
        constant: 0,
    };
    let sb = Switchbox {
        output_wires: (0, 0),
    };
    let functional_units: [[FunctionalUnit; 8]; 8] = [[functional_unit; 8]; 8];

    let slices = [SliceDesc {
        functional_units: functional_units,
        switchboxes: [[sb; 8]; 8],
    }; 1];

    ChipDesc { slices }
}
// Grape stuff
#[no_mangle]
pub unsafe extern "C" fn program_bitstream(ptr: *const u8, len: usize) {
    // Example: read the bytes from the pointer
    if ptr.is_null() {
        panic!("Null pointer passed to program_bitstream");
    }

    let bytes = std::slice::from_raw_parts(ptr, len);
    let chip_desc: SliceDesc<8, 8> = serde_json::from_slice(bytes).expect("Failed to parse JSON");

    unsafe {
        CHIP.slices[FRAME_IDX] = chip_desc;
    }
    // println!("Received bitstream of length {}", slice.len());

    // for (i, byte) in slice.iter().enumerate() {
    //     println!("Byte {}: {:08b}", i, byte);
    // }
}

pub fn compute() -> [i16; 8] {
    let mut values: [[i16; 8]; 8] = [[0; 8]; 8];
    for row in 0..8 {
        for col in 0..8 {
            unsafe {
                let op = CHIP.slices[FRAME_IDX].functional_units[row][col].op_type;

                let ins = if row == 0 {
                    (&INPUT_DATA[col * 2], &INPUT_DATA[col * 2 + 1])
                } else {
                    let ins = CHIP.slices[FRAME_IDX].switchboxes[row - 1][col].output_wires;

                    let ins = (
                        values[row - 1].get(ins.0).unwrap_or(&0),
                        values[row - 1].get(ins.1).unwrap_or(&0),
                    );
                    ins
                };

                println!("row: {}, col: {}, ins: {:?} op: {:?}", row, col, ins, op);

                let result = match op {
                    hercules_cg::FuOp::Add => ins.0.wrapping_add(*ins.1),
                    hercules_cg::FuOp::Sub => ins.0.wrapping_sub(*ins.1),
                    hercules_cg::FuOp::Mult => ins.0 * ins.1,
                    hercules_cg::FuOp::PassA => *ins.0,
                    hercules_cg::FuOp::Default => {
                        CHIP.slices[FRAME_IDX].functional_units[row][col].constant
                    }
                };
                values[row][col] = result;
            }
        }
    }

    return values[7];
}

#[no_mangle]
pub unsafe extern "C" fn send_data(inputs: *const i16) {
    // panic!();
    for i in 0..16 {
        INPUT_DATA[i] = *inputs.add(i);
    }
    println!("inputs: {:?}", INPUT_DATA);
} // len is 16

#[no_mangle]
pub unsafe extern "C" fn await_valid() {}

#[no_mangle]
pub unsafe extern "C" fn read_data(outputs: *mut i16) {
    // This forces the computation.
    let out = compute();
    print!("out: {:?}\n", out);
    for i in 0..7 {
        *outputs.add(i) = out[i];
    }
} // len is 7
