#![feature(concat_idents)]

use std::time::Instant;

#[cfg(feature = "cuda")]
use hercules_rt::CUDABox;

use hercules_rt::{runner, HerculesCPURef};

juno_build::juno!("conv");

#[cfg(feature = "grape")]
use grape_sim::*;

fn conv1d<const N: usize, const K: usize>(a: &[i16; N], kernel: &[i16; K]) -> [i16; N] {
    let mut res = [0i16; N];

    for i in 0..N {
        let mut window_left = i as i64 - (K as i64) / 2;
        let mut window_right = i as i64 + (K as i64) / 2;

        if window_left < 0 {
            window_left = 0;
        }
        if window_right >= N as i64 {
            window_right = N as i64 - 1;
        }

        let mut acc: i16 = 0;
        for j in 0..(window_right - window_left + 1) as usize {
            let inc_j = j + window_left as usize;
            acc += a[inc_j] * kernel[j];
        }
        res[i] = acc;
    }

    res
}

fn main() {
    async_std::task::block_on(async {
        let a: Box<[i16]> = Box::new([1, 2, 3, 4]);
        let b: Box<[i16]> = Box::new([5, 6, 7]);

        let result: [i16; 4] = conv1d::<4, 3>(
            a.as_ref().try_into().unwrap(),
            b.as_ref().try_into().unwrap(),
        );

        #[cfg(not(feature = "cuda"))]
        {
            let a = HerculesCPURef::from_slice(&a);
            let b = HerculesCPURef::from_slice(&b);
            let start = Instant::now();
            let mut r = runner!(entry);
            let c = r.run(a, b).await;
            print!("{:?}", c);
            let duration = start.elapsed();
            println!("Time elapsed in some_function() is: {:?}", duration);

            assert!(false);
            assert_eq!(c, (result[0], result[1], result[2], result[3]));
        }
        #[cfg(feature = "cuda")]
        {
            let a = CUDABox::from_cpu_ref(HerculesCPURef::from_slice(&a));
            let b = CUDABox::from_cpu_ref(HerculesCPURef::from_slice(&b));
            let mut r = runner!(simple);
            let c = r.run(8, a.get_ref(), b.get_ref()).await;
            assert_eq!(c, 120);
        }
    });
}

#[test]
fn simple3_test() {
    main();
}
