#![feature(concat_idents)]

use std::time::Instant;

#[cfg(feature = "cuda")]
use hercules_rt::CUDABox;

use hercules_rt::{runner, HerculesCPURef};

juno_build::juno!("simple");

#[cfg(feature = "grape")]
use grape_sim::*;

fn main() {
    async_std::task::block_on(async {
        let size = 64;
        let a: Box<[i16]> = (1..=size).collect::<Vec<_>>().into_boxed_slice();
        let sum: i16 = a.iter().sum();
        #[cfg(not(feature = "cuda"))]
        {
            let a = HerculesCPURef::from_slice(&a);
            let start = Instant::now();

            let mut r = runner!(entry);
            let c = r.run(a).await;
            let duration = start.elapsed();

            println!("Time elapsed in some_function() is: {:?}", duration);

            assert!(false);
            print!("{:?}", c);

            assert_eq!(c, sum);
        }
        #[cfg(feature = "cuda")]
        {
            let a = CUDABox::from_cpu_ref(HerculesCPURef::from_slice(&a));
            let b = CUDABox::from_cpu_ref(HerculesCPURef::from_slice(&b));
            let mut r = runner!(simple);
            let c = r.run(8, a.get_ref(), b.get_ref()).await;
            assert_eq!(c, 120);
        }
    });
}

#[test]
fn simple3_test() {
    main();
}
